@@ -45,7 +45,6 @@
 #include "screenshot-utils.h"
 #include "screenshot-save.h"
 #include "screenshot-dialog.h"
-#include "screenshot-xfer.h"
 #include "cheese-flash.h"
 
 #define SCREENSHOOTER_ICON "applets-screenshooter"
@@ -99,8 +98,10 @@ static char *temporary_file = NULL;
 static gboolean save_immediately = FALSE;
 static GSettings *settings = NULL;
 static CheeseFlash *flash = NULL;
+static GDBusConnection *connection = NULL;
 
 /* Options */
+static gboolean copy_to_clipboard = FALSE;
 static gboolean take_window_shot = FALSE;
 static gboolean take_area_shot = FALSE;
 static gboolean include_border = FALSE;
@@ -610,17 +611,49 @@ error_dialog_response_cb (GtkDialog *d,
 }
 
 static void
-save_callback (TransferResult result,
-               char *error_message,
-               gpointer data)
+save_callback (GObject *source,
+               GAsyncResult *res,
+               gpointer user_data)
 {
-  ScreenshotDialog *dialog = data;
+  ScreenshotDialog *dialog = user_data;
   GtkWidget *toplevel;
+  GError *error = NULL;
   
   toplevel = screenshot_dialog_get_toplevel (dialog);
   screenshot_dialog_set_busy (dialog, FALSE);
 
-  if (result == TRANSFER_OK)
+  g_dbus_connection_call_finish (connection, res, &error);
+
+  if (error != NULL)
+    {
+      /* we had an error, display a dialog to the user and let him choose
+       * another name/location to save the screenshot.
+       */
+      GtkWidget *error_dialog;
+      char *folder;
+      
+      folder = screenshot_dialog_get_folder (dialog);
+      error_dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
+                                       GTK_DIALOG_DESTROY_WITH_PARENT,
+                                       GTK_MESSAGE_ERROR,
+                                       GTK_BUTTONS_OK,
+                                       _("Error while saving screenshot"));
+
+      /* translators: first %s is the folder URI, second %s is the VFS error */
+      gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error_dialog),
+                                                _("Impossible to save the screenshot "
+                                                  "to %s.\n Error was %s.\n Please choose another "
+                                                  "location and retry."), folder, error->message);
+      gtk_widget_show (error_dialog);
+      g_signal_connect (error_dialog,
+                        "response",
+                        G_CALLBACK (error_dialog_response_cb),
+                        dialog);
+
+      g_free (folder);
+      g_error_free (error);
+    }
+  else
     {
       save_folder_to_settings (dialog);
       set_recent_entry (dialog);
@@ -629,65 +662,46 @@ save_callback (TransferResult result,
       /* we're done, stop the mainloop now */
       gtk_main_quit ();
     }
-  else if (result == TRANSFER_OVERWRITE ||
-           result == TRANSFER_CANCELLED)
-    {
-      /* user has canceled the overwrite dialog or the transfer itself, let him
-       * choose another name.
-       */
-      screenshot_dialog_focus_entry (dialog);
-    }
-  else /* result == TRANSFER_ERROR */
-    {
-      /* we had an error, display a dialog to the user and let him choose
-       * another name/location to save the screenshot.
-       */
-      GtkWidget *error_dialog;
-      char *uri;
-      
-      uri = screenshot_dialog_get_uri (dialog);
-      error_dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-                                       GTK_DIALOG_DESTROY_WITH_PARENT,
-                                       GTK_MESSAGE_ERROR,
-                                       GTK_BUTTONS_OK,
-                                       _("Error while saving screenshot"));
-      /* translators: first %s is the file path, second %s is the VFS error */
-      gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error_dialog),
-                                                _("Impossible to save the screenshot "
-                                                  "to %s.\n Error was %s.\n Please choose another "
-                                                  "location and retry."), uri, error_message);
-      gtk_widget_show (error_dialog);
-      g_signal_connect (error_dialog,
-                        "response",
-                        G_CALLBACK (error_dialog_response_cb),
-                        dialog);
-
-      g_free (uri);
-    }
-      
 }
 
 static void
-try_to_save (ScreenshotDialog *dialog,
-	     const char       *target)
+try_to_save (ScreenshotDialog *dialog)
 {
-  GFile *source_file, *target_file;
+  gchar *target_folder, *target_filename, *source_uri;
+  GFile *source_file;
 
   g_assert (temporary_file);
 
   screenshot_dialog_set_busy (dialog, TRUE);
 
   source_file = g_file_new_for_path (temporary_file);
-  target_file = g_file_new_for_uri (target);
-  
-  screenshot_xfer_uri (source_file,
-                       target_file,
-                       screenshot_dialog_get_toplevel (dialog),
-                       save_callback, dialog);
-  
-  /* screenshot_xfer_uri () holds a ref, so we can unref now */
+  source_uri = g_file_get_uri (source_file);
+
+  target_folder = screenshot_dialog_get_folder (dialog);
+  target_filename = screenshot_dialog_get_filename (dialog);
+
+  g_dbus_connection_call (connection,
+                          "org.gnome.Nautilus",
+                          "/org/gnome/Nautilus",
+                          "org.gnome.Nautilus.FileOperations",
+                          "CopyFile",
+                          g_variant_new ("(ssss)",
+                                         source_uri,
+                                         target_filename,
+                                         target_folder,
+                                         target_filename),
+                          NULL,
+                          G_DBUS_CALL_FLAGS_NONE,
+                          -1,
+                          NULL,
+                          save_callback,
+                          dialog);
+
   g_object_unref (source_file);
-  g_object_unref (target_file);
+
+  g_free (source_uri);
+  g_free (target_folder);
+  g_free (target_filename);
 }
 
 static void
@@ -712,15 +726,12 @@ screenshot_dialog_response_cb (GtkDialog *d,
                                gint response_id,
                                ScreenshotDialog *dialog)
 {
-  char *uri;
-
   if (response_id == GTK_RESPONSE_HELP)
     {
       display_help (GTK_WINDOW (d));
     }
   else if (response_id == GTK_RESPONSE_OK)
     {
-      uri = screenshot_dialog_get_uri (dialog);
       if (temporary_file == NULL)
         {
           save_immediately = TRUE;
@@ -731,9 +742,8 @@ screenshot_dialog_response_cb (GtkDialog *d,
           /* we've saved the temporary file, lets try to copy it to the
            * correct location.
            */
-          try_to_save (dialog, uri);
+          try_to_save (dialog);
         }
-      g_free (uri);
     }
   else if (response_id == SCREENSHOT_RESPONSE_COPY)
     {
@@ -811,7 +821,6 @@ static gchar *
 get_profile_for_window (GdkWindow *window)
 {
   gboolean ret = FALSE;
-  GDBusConnection *connection;
   GError *error = NULL;
   guint xid;
   gchar *icc_profile = NULL;
@@ -820,15 +829,6 @@ get_profile_for_window (GdkWindow *window)
   GVariant *response_child = NULL;
   GVariantIter *iter = NULL;
 
-  /* get a session bus connection */
-  connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
-  if (connection == NULL)
-    {
-      g_debug ("Failed to connect to session bus: %s", error->message);
-      g_error_free (error);
-      goto out;
-    }
-
   /* get color profile */
   xid = GDK_WINDOW_XID (window);
   args = g_variant_new ("(u)", xid),
@@ -917,42 +917,54 @@ finish_prepare_screenshot (char *initial_uri, GdkWindow *window, GdkRectangle *r
   cheese_flash_fire (flash, &rect);
   play_sound_effect (window);
 
-  dialog = screenshot_dialog_new (screenshot, initial_uri, take_window_shot);
-  g_free (initial_uri);
-
-  /* load ICC profile */
-  if (include_icc_profile)
+  if (copy_to_clipboard)
     {
-      icc_profile_filename = get_profile_for_window (window);
-      if (icc_profile_filename != NULL)
-        {
-          ret = g_file_get_contents (icc_profile_filename,
-                                     (gchar **) &icc_profile,
-                                     &icc_profile_size,
-                                     &error);
-          if (ret)
-            {
-              icc_profile_base64 = g_base64_encode (icc_profile,
-                                                    icc_profile_size);
+      GtkClipboard *clipboard;
 
-              /* use this profile for saving the image */
-              screenshot_set_icc_profile (icc_profile_base64);
-              g_free (icc_profile);
-              g_free (icc_profile_base64);
-            }
-          else
+      clipboard = gtk_clipboard_get_for_display (gdk_display_get_default (),
+                                                 GDK_SELECTION_CLIPBOARD);
+      gtk_clipboard_set_image (clipboard, screenshot);
+      gtk_main_quit ();
+    }
+  else
+    {
+      /* load ICC profile */
+      if (include_icc_profile)
+        {
+          icc_profile_filename = get_profile_for_window (window);
+          if (icc_profile_filename != NULL)
             {
-              g_warning ("could not open ICC file: %s",
-                         error->message);
-              g_error_free (error);
+              ret = g_file_get_contents (icc_profile_filename,
+                                         (gchar **) &icc_profile,
+                                         &icc_profile_size,
+                                         &error);
+              if (ret)
+                {
+                  icc_profile_base64 = g_base64_encode (icc_profile,
+                                                        icc_profile_size);
+
+                  /* use this profile for saving the image */
+                  screenshot_set_icc_profile (icc_profile_base64);
+                  g_free (icc_profile);
+                  g_free (icc_profile_base64);
+                }
+              else
+                {
+                  g_warning ("could not open ICC file: %s",
+                             error->message);
+                  g_error_free (error);
+                }
+              g_free (icc_profile_filename);
             }
-          g_free (icc_profile_filename);
         }
+      dialog = screenshot_dialog_new (screenshot, initial_uri, take_window_shot);
+
+      screenshot_save_start (screenshot, save_done_notification, dialog);
+
+      run_dialog (dialog);
     }
 
-  screenshot_save_start (screenshot, save_done_notification, dialog);
-
-  run_dialog (dialog);
+  g_free (initial_uri);
 }
 
 static void
@@ -1328,11 +1340,29 @@ screenshooter_init_stock_icons (void)
   g_object_unref (factory);
 }
 
+static void
+init_dbus_session (void)
+{
+  GError *error = NULL;
+
+  connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
+
+  if (error != NULL)
+    {
+      g_critical ("Unable to connect to the session bus: %s",
+                  error->message);
+      g_error_free (error);
+
+      exit (1);
+    }
+}
+
 /* main */
 int
 main (int argc, char *argv[])
 {
   GOptionContext *context;
+  gboolean clipboard_arg = FALSE;
   gboolean window_arg = FALSE;
   gboolean area_arg = FALSE;
   gboolean include_border_arg = FALSE;
@@ -1343,6 +1373,7 @@ main (int argc, char *argv[])
   GError *error = NULL;
 
   const GOptionEntry entries[] = {
+    { "clipboard", 'c', 0, G_OPTION_ARG_NONE, &clipboard_arg, N_("Send the grab directly to the clipboard"), NULL },
     { "window", 'w', 0, G_OPTION_ARG_NONE, &window_arg, N_("Grab a window instead of the entire screen"), NULL },
     { "area", 'a', 0, G_OPTION_ARG_NONE, &area_arg, N_("Grab an area of the screen instead of the entire screen"), NULL },
     { "include-border", 'b', 0, G_OPTION_ARG_NONE, &include_border_arg, N_("Include the window border with the screenshot"), NULL },
@@ -1386,6 +1417,8 @@ main (int argc, char *argv[])
   gtk_window_set_default_icon_name (SCREENSHOOTER_ICON);
   screenshooter_init_stock_icons ();
 
+  init_dbus_session ();
+
   settings = g_settings_new ("org.gnome.gnome-screenshot");
   load_options ();
   /* allow the command line to override options */
@@ -1410,6 +1443,9 @@ main (int argc, char *argv[])
   if (delay_arg > 0)
     delay = delay_arg;
 
+  if (clipboard_arg)
+    copy_to_clipboard = TRUE;
+
   /* interactive mode overrides everything */
   if (interactive_arg)
     {
@@ -1457,8 +1493,8 @@ main (int argc, char *argv[])
 
   gtk_main ();
 
-  if (flash != NULL)
-    g_object_unref (flash);
+  g_clear_object (&flash);
+  g_clear_object (&connection);
 
   return EXIT_SUCCESS;
 }
